@@ -6,12 +6,15 @@ import sys
 import traceback
 import json
 import os
+import re
+import itertools
 
+import six
 from six.moves import urllib
 
 import pkg_resources  # part of setuptools
 
-from typing import Any, Dict, List, Union, Text
+from typing import Any, Dict, List, Union, Pattern, Text, Tuple, cast
 
 from rdflib import Graph, plugin
 from rdflib.serializer import Serializer
@@ -20,10 +23,9 @@ from . import schema
 from . import jsonld_context
 from . import makedoc
 from . import validate
+from . import codegen
 from .sourceline import strip_dup_lineno
-from .ref_resolver import Loader
-import six
-
+from .ref_resolver import Loader, file_uri
 _logger = logging.getLogger("salad")
 
 from rdflib.plugin import register, Parser
@@ -40,6 +42,82 @@ def printrdf(workflow,  # type: str
     print(g.serialize(format=sr))
 
 
+def regex_chunk(lines, regex):
+    # type: (List[str], Pattern[str]) -> List[List[str]]
+    lst = list(itertools.dropwhile(lambda x: not regex.match(x), lines))
+    arr = []
+    while lst:
+        ret = [lst[0]]+list(itertools.takewhile(lambda x: not regex.match(x),
+                                                lst[1:]))
+        arr.append(ret)
+        lst = list(itertools.dropwhile(lambda x: not regex.match(x),
+                                       lst[1:]))
+    return arr
+
+
+def chunk_messages(message):  # type: (str) -> List[Tuple[int, str]]
+    file_regex = re.compile(r'^(.+:\d+:\d+:)(\s+)(.+)$')
+    item_regex = re.compile(r'^\s*\*\s+')
+    arr = []
+    for chun in regex_chunk(message.splitlines(), file_regex):
+        fst = chun[0]
+        mat = file_regex.match(fst)
+        place = mat.group(1)
+        indent = len(mat.group(2))
+
+        lst = [mat.group(3)]+chun[1:]
+        if [x for x in lst if item_regex.match(x)]:
+            for item in regex_chunk(lst, item_regex):
+                msg = re.sub(item_regex, '', "\n".join(item))
+                arr.append((indent, place+' '+re.sub(r'[\n\s]+',
+                                                     ' ',
+                                                     msg)))
+        else:
+            msg = re.sub(item_regex, '', "\n".join(lst))
+            arr.append((indent, place+' '+re.sub(r'[\n\s]+',
+                                                 ' ',
+                                                 msg)))
+    return arr
+
+
+def to_one_line_messages(message):  # type: (str) -> str
+    ret = []
+    max_elem = (0, '')
+    for (indent, msg) in chunk_messages(message):
+        if indent > max_elem[0]:
+            max_elem = (indent, msg)
+        else:
+            ret.append(max_elem[1])
+            max_elem = (indent, msg)
+    ret.append(max_elem[1])
+    return "\n".join(ret)
+
+
+def reformat_yaml_exception_message(message):  # type: (str) -> str
+    line_regex = re.compile(r'^\s+in "(.+)", line (\d+), column (\d+)$')
+    fname_regex = re.compile(r'^file://'+os.getcwd()+'/')
+    msgs = message.splitlines()
+    ret = []
+
+    if len(msgs) == 3:
+        msgs = msgs[1:]
+        nblanks = 0
+    elif len(msgs) == 4:
+        c_msg = msgs[0]
+        c_file, c_line, c_column = line_regex.match(msgs[1]).groups()
+        c_file = re.sub(fname_regex, '', c_file)
+        ret.append("%s:%s:%s: %s" % (c_file, c_line, c_column, c_msg))
+
+        msgs = msgs[2:]
+        nblanks = 2
+
+    p_msg = msgs[0]
+    p_file, p_line, p_column = line_regex.match(msgs[1]).groups()
+    p_file = re.sub(fname_regex, '', p_file)
+    ret.append("%s:%s:%s:%s %s" % (p_file, p_line, p_column, ' '*nblanks, p_msg))
+    return "\n".join(ret)
+
+
 def main(argsl=None):  # type: (List[str]) -> int
     if argsl is None:
         argsl = sys.argv[1:]
@@ -66,6 +144,11 @@ def main(argsl=None):  # type: (List[str]) -> int
     exgroup.add_argument("--print-metadata",
                          action="store_true", help="Print document metadata")
 
+    exgroup.add_argument("--codegen", type=str, metavar="language", help="Generate classes in target language, currently supported: python")
+
+    exgroup.add_argument("--print-oneline", action="store_true",
+                         help="Print each error message in oneline")
+
     exgroup = parser.add_mutually_exclusive_group()
     exgroup.add_argument("--strict", action="store_true", help="Strict validation (unrecognized or out of place fields are error)",
                          default=True, dest="strict")
@@ -111,8 +194,8 @@ def main(argsl=None):  # type: (List[str]) -> int
     # Load schema document and resolve refs
 
     schema_uri = args.schema
-    if not urllib.parse.urlparse(schema_uri)[0]:
-        schema_uri = "file://" + os.path.abspath(schema_uri)
+    if not (urllib.parse.urlparse(schema_uri)[0] and urllib.parse.urlparse(schema_uri)[0] in [u'http', u'https', u'file']):
+        schema_uri = file_uri(os.path.abspath(schema_uri))
     schema_raw_doc = metaschema_loader.fetch(schema_uri)
 
     try:
@@ -163,6 +246,11 @@ def main(argsl=None):  # type: (List[str]) -> int
     # Create the loader that will be used to load the target document.
     document_loader = Loader(schema_ctx)
 
+    if args.codegen:
+        codegen.codegen(args.codegen, cast(List[Dict[Text, Any]], schema_doc),
+                        schema_metadata, document_loader)
+        return 0
+
     # Make the Avro validation that will be used to validate the target
     # document
     if isinstance(schema_doc, list):
@@ -211,9 +299,18 @@ def main(argsl=None):  # type: (List[str]) -> int
         if not urllib.parse.urlparse(uri)[0]:
             doc = "file://" + os.path.abspath(uri)
         document, doc_metadata = document_loader.resolve_ref(uri)
-    except (validate.ValidationException, RuntimeError) as e:
+    except validate.ValidationException as e:
+        msg = strip_dup_lineno(six.text_type(e))
+        msg = to_one_line_messages(str(msg)) if args.print_oneline else msg
         _logger.error("Document `%s` failed validation:\n%s",
-                      args.document, strip_dup_lineno(six.text_type(e)), exc_info=args.debug)
+                      args.document, msg, exc_info=args.debug)
+        return 1
+    except RuntimeError as e:
+        msg = strip_dup_lineno(six.text_type(e))
+        msg = reformat_yaml_exception_message(str(msg))
+        msg = to_one_line_messages(msg) if args.print_oneline else msg
+        _logger.error("Document `%s` failed validation:\n%s",
+                      args.document, msg, exc_info=args.debug)
         return 1
 
     # Optionally print the document after ref resolution
@@ -230,8 +327,9 @@ def main(argsl=None):  # type: (List[str]) -> int
         schema.validate_doc(avsc_names, document,
                             document_loader, args.strict)
     except validate.ValidationException as e:
+        msg = to_one_line_messages(str(e)) if args.print_oneline else str(e)
         _logger.error("While validating document `%s`:\n%s" %
-                      (args.document, str(e)))
+                      (args.document, msg))
         return 1
 
     # Optionally convert the document to RDF
