@@ -5,6 +5,7 @@ import json
 import hashlib
 import logging
 import collections
+from io import open
 
 import six
 from six.moves import range
@@ -40,6 +41,8 @@ DocumentType = TypeVar('DocumentType', CommentedSeq, CommentedMap)
 DocumentOrStrType = TypeVar(
     'DocumentOrStrType', CommentedSeq, CommentedMap, six.text_type)
 
+_re_drive = re.compile(r"/([a-zA-Z]):")
+
 def file_uri(path, split_frag=False):  # type: (str, bool) -> str
     if path.startswith("file://"):
         return path
@@ -117,7 +120,7 @@ class Fetcher(object):
 
 class DefaultFetcher(Fetcher):
     def __init__(self,
-                 cache,   # type: Dict[Text, Text]
+                 cache,   # type: Dict[Text, Union[Text, bool]]
                  session  # type: Optional[requests.sessions.Session]
                  ):  # type: (...) -> None
         self.cache = cache
@@ -125,8 +128,10 @@ class DefaultFetcher(Fetcher):
 
     def fetch_text(self, url):
         # type: (Text) -> Text
-        if url in self.cache:
-            return self.cache[url]
+        if url in self.cache and self.cache[url] is not True:
+            # treat "True" as a placeholder that indicates something exists but
+            # not necessarily what its contents is.
+            return cast(Text, self.cache[url])
 
         split = urllib.parse.urlsplit(url)
         scheme, path = split.scheme, split.path
@@ -140,12 +145,14 @@ class DefaultFetcher(Fetcher):
             return resp.text
         elif scheme == 'file':
             try:
-                with open(urllib.request.url2pathname(str(path))) as fp:
-                    read = fp.read()
-                if hasattr(read, "decode"):
-                    return read.decode("utf-8")
-                else:
-                    return read
+                # On Windows, url.path will be /drive:/path ; on Unix systems,
+                # /path. As we want drive:/path instead of /drive:/path on Windows,
+                # remove the leading /.
+                if os.path.isabs(path[1:]):  # checking if pathis valid after removing front / or not
+                    path = path[1:]
+                with open(urllib.request.url2pathname(str(path)), encoding='utf-8') as fp:
+                    return fp.read()
+
             except (OSError, IOError) as e:
                 if e.filename == path:
                     raise RuntimeError(six.text_type(e))
@@ -167,6 +174,7 @@ class DefaultFetcher(Fetcher):
                 resp.raise_for_status()
             except Exception as e:
                 return False
+            self.cache[url] = True
             return True
         elif scheme == 'file':
             return os.path.exists(urllib.request.url2pathname(str(path)))
@@ -174,6 +182,61 @@ class DefaultFetcher(Fetcher):
             raise ValueError('Unsupported scheme in url: %s' % url)
 
     def urljoin(self, base_url, url):  # type: (Text, Text) -> Text
+        basesplit = urllib.parse.urlsplit(base_url)
+        split = urllib.parse.urlsplit(url)
+        if (basesplit.scheme and basesplit.scheme != "file" and split.scheme == "file"):
+            raise ValueError("Not resolving potential remote exploit %s from base %s" % (url, base_url))
+
+        if sys.platform == 'win32':
+            if (base_url == url):
+                return url
+            basesplit = urllib.parse.urlsplit(base_url)
+            # note that below might split
+            # "C:" with "C" as URI scheme
+            split = urllib.parse.urlsplit(url)
+
+            has_drive = split.scheme and len(split.scheme) == 1
+
+            if basesplit.scheme == "file":
+                # Special handling of relative file references on Windows
+                # as urllib seems to not be quite up to the job
+
+                # netloc MIGHT appear in equivalents of UNC Strings
+                # \\server1.example.com\path as
+                # file:///server1.example.com/path
+                # https://tools.ietf.org/html/rfc8089#appendix-E.3.2
+                # (TODO: test this)
+                netloc = split.netloc or basesplit.netloc
+
+                # Check if url is a local path like "C:/Users/fred"
+                # or actually an absolute URI like http://example.com/fred
+                if has_drive:
+                    # Assume split.scheme is actually a drive, e.g. "C:"
+                    # so we'll recombine into a path
+                    path_with_drive = urllib.parse.urlunsplit((split.scheme, '', split.path,'', ''))
+                    # Compose new file:/// URI with path_with_drive
+                    # .. carrying over any #fragment (?query just in case..)
+                    return urllib.parse.urlunsplit(("file", netloc,
+                                        path_with_drive, split.query, split.fragment))
+                if (not split.scheme and not netloc and
+                    split.path and split.path.startswith("/")):
+                    # Relative - but does it have a drive?
+                    base_drive = _re_drive.match(basesplit.path)
+                    drive = _re_drive.match(split.path)
+                    if base_drive and not drive:
+                        # Keep drive letter from base_url
+                        # https://tools.ietf.org/html/rfc8089#appendix-E.2.1
+                        # e.g. urljoin("file:///D:/bar/a.txt", "/foo/b.txt") == file:///D:/foo/b.txt
+                        path_with_drive = "/%s:%s" % (base_drive.group(1), split.path)
+                        return urllib.parse.urlunsplit(("file", netloc, path_with_drive,
+                                                   split.query, split.fragment))
+
+                # else: fall-through to resolve as relative URI
+            elif has_drive:
+                # Base is http://something but url is C:/something - which urllib would wrongly
+                # resolve as an absolute path that could later be used to access local files
+                raise ValueError("Not resolving potential remote exploit %s from base %s" % (url, base_url))
+
         return urllib.parse.urljoin(base_url, url)
 
 class Loader(object):
@@ -184,7 +247,7 @@ class Loader(object):
                  idx=None,                  # type: Dict[Text, Union[CommentedMap, CommentedSeq, Text, None]]
                  cache=None,                # type: Dict[Text, Any]
                  session=None,              # type: requests.sessions.Session
-                 fetcher_constructor=None,  # type: Callable[[Dict[Text, Text], requests.sessions.Session], Fetcher]
+                 fetcher_constructor=None,  # type: Callable[[Dict[Text, Union[Text, bool]], requests.sessions.Session], Fetcher]
                  skip_schemas=None          # type: bool
                  ):
         # type: (...) -> None
@@ -237,7 +300,6 @@ class Loader(object):
         else:
             self.fetcher_constructor = DefaultFetcher
         self.fetcher = self.fetcher_constructor(self.cache, self.session)
-
         self.fetch_text = self.fetcher.fetch_text
         self.check_exists = self.fetcher.check_exists
 
@@ -277,7 +339,7 @@ class Loader(object):
 
         split = urllib.parse.urlsplit(url)
 
-        if (bool(split.scheme) or url.startswith(u"$(")
+        if ((bool(split.scheme) and split.scheme in [u'http', u'https', u'file']) or url.startswith(u"$(")
             or url.startswith(u"${")):
             pass
         elif scoped_id and not bool(split.fragment):
@@ -319,22 +381,25 @@ class Loader(object):
         if self.skip_schemas:
             return
         for sch in aslist(ns):
-            fetchurl = self.fetcher.urljoin(base_url, sch)
-            if fetchurl not in self.cache:
-                _logger.debug("Getting external schema %s", fetchurl)
-                content = self.fetch_text(fetchurl)
-                self.cache[fetchurl] = rdflib.graph.Graph()
-                for fmt in ['xml', 'turtle', 'rdfa']:
-                    try:
-                        self.cache[fetchurl].parse(data=content, format=fmt, publicID=str(fetchurl))
-                        self.graph += self.cache[fetchurl]
-                        break
-                    except xml.sax.SAXParseException:
-                        pass
-                    except TypeError:
-                        pass
-                    except BadSyntax:
-                        pass
+            try:
+                fetchurl = self.fetcher.urljoin(base_url, sch)
+                if fetchurl not in self.cache or self.cache[fetchurl] is True:
+                    _logger.debug("Getting external schema %s", fetchurl)
+                    content = self.fetch_text(fetchurl)
+                    self.cache[fetchurl] = rdflib.graph.Graph()
+                    for fmt in ['xml', 'turtle', 'rdfa']:
+                        try:
+                            self.cache[fetchurl].parse(data=content, format=fmt, publicID=str(fetchurl))
+                            self.graph += self.cache[fetchurl]
+                            break
+                        except xml.sax.SAXParseException:
+                            pass
+                        except TypeError:
+                            pass
+                        except BadSyntax:
+                            pass
+            except Exception as e:
+                _logger.warn("Could not load extension schema %s: %s", fetchurl, e)
 
         for s, _, _ in self.graph.triples((None, RDF.type, RDF.Property)):
             self._add_properties(s)
@@ -430,10 +495,6 @@ class Loader(object):
         if not base_url:
             base_url = file_uri(os.getcwd()) + "/"
 
-        if isinstance(lref, (str, six.text_type)) and os.sep == "\\":
-            # Convert Windows path separator in ref
-            lref = lref.replace("\\", "/")
-
         sl = SourceLine(obj, None, ValueError)
         # If `ref` is a dict, look for special directives.
         if isinstance(lref, CommentedMap):
@@ -477,6 +538,10 @@ class Loader(object):
             raise ValueError(u"Expected CommentedMap or string, got %s: `%s`"
                     % (type(lref), six.text_type(lref)))
 
+        if isinstance(lref, (str, six.text_type)) and os.sep == "\\":
+            # Convert Windows path separator in ref
+            lref = lref.replace("\\", "/")
+
         url = self.expand_url(lref, base_url, scoped_id=(obj is not None))
         # Has this reference been loaded already?
         if url in self.idx and (not mixin):
@@ -966,7 +1031,10 @@ class Loader(object):
                             all_doc_ids[document[identifier]] = sl.makeLead()
                             break
             except validate.ValidationException as v:
-                errors.append(sl.makeError(six.text_type(v)))
+                if d == "$schemas":
+                    _logger.warn( validate.indent(six.text_type(v)))
+                else:
+                    errors.append(sl.makeError(six.text_type(v)))
             if hasattr(document, "iteritems"):
                 iterator = six.iteritems(document)
             else:
@@ -979,7 +1047,9 @@ class Loader(object):
             try:
                 self.validate_links(val, docid, all_doc_ids)
             except validate.ValidationException as v:
-                if key not in self.nolinkcheck:
+                if key in self.nolinkcheck or (isinstance(key, six.string_types) and ":" in key):
+                    _logger.warn( validate.indent(six.text_type(v)))
+                else:
                     docid2 = self.getid(val)
                     if docid2 is not None:
                         errors.append(sl.makeError("checking object `%s`\n%s"
@@ -991,8 +1061,6 @@ class Loader(object):
                         else:
                             errors.append(sl.makeError("checking item\n%s" % (
                                 validate.indent(six.text_type(v)))))
-                else:
-                    _logger.warn( validate.indent(six.text_type(v)))
         if bool(errors):
             if len(errors) > 1:
                 raise validate.ValidationException(
