@@ -10,7 +10,7 @@ import rdflib
 import ruamel.yaml
 import json
 import os
-from schema_salad.sourceline import cmap
+from schema_salad.sourceline import cmap, SourceLine
 
 try:
     from ruamel.yaml import CSafeLoader as SafeLoader
@@ -22,16 +22,16 @@ from ruamel.yaml.comments import CommentedSeq, CommentedMap
 
 class TestSchemas(unittest.TestCase):
     def test_schemas(self):
-        l = schema_salad.ref_resolver.Loader({})
+        loader = schema_salad.ref_resolver.Loader({})
 
-        ra, _ = l.resolve_all(cmap({
-            u"$schemas": ["file://" + get_data("tests/EDAM.owl")],
+        ra, _ = loader.resolve_all(cmap({
+            u"$schemas": [schema_salad.ref_resolver.file_uri(get_data("tests/EDAM.owl"))],
             u"$namespaces": {u"edam": u"http://edamontology.org/"},
             u"edam:has_format": u"edam:format_1915"
         }), "")
 
         self.assertEqual({
-            u"$schemas": ["file://" + get_data("tests/EDAM.owl")],
+            u"$schemas": [schema_salad.ref_resolver.file_uri(get_data("tests/EDAM.owl"))],
             u"$namespaces": {u"edam": u"http://edamontology.org/"},
             u'http://edamontology.org/has_format': u'http://edamontology.org/format_1915'
         }, ra)
@@ -332,7 +332,7 @@ class TestSchemas(unittest.TestCase):
         print(g.serialize(format="n3"))
 
     def test_mixin(self):
-        base_url = "file://" + os.getcwd() + "/tests/"
+        base_url = schema_salad.ref_resolver.file_uri(os.path.join(os.getcwd(), "tests"))
         ldr = schema_salad.ref_resolver.Loader({})
         ra = ldr.resolve_ref(cmap({"$mixin": get_data("tests/mixin.yml"), "one": "five"}),
                              base_url=base_url)
@@ -367,11 +367,38 @@ class TestSchemas(unittest.TestCase):
     def test_file_uri(self):
         # Note: this test probably won't pass on Windows.  Someone with a
         # windows box should add an alternate test.
-        self.assertEqual("file:///foo/bar%20baz/quux", schema_salad.ref_resolver.file_uri("/foo/bar baz/quux"))
-        self.assertEqual("/foo/bar baz/quux", schema_salad.ref_resolver.uri_file_path("file:///foo/bar%20baz/quux"))
-        self.assertEqual("file:///foo/bar%20baz/quux%23zing%20zong", schema_salad.ref_resolver.file_uri("/foo/bar baz/quux#zing zong"))
-        self.assertEqual("file:///foo/bar%20baz/quux#zing%20zong", schema_salad.ref_resolver.file_uri("/foo/bar baz/quux#zing zong", split_frag=True))
-        self.assertEqual("/foo/bar baz/quux#zing zong", schema_salad.ref_resolver.uri_file_path("file:///foo/bar%20baz/quux#zing%20zong"))
+        self.assertEquals("file:///foo/bar%20baz/quux", schema_salad.ref_resolver.file_uri("/foo/bar baz/quux"))
+        self.assertEquals(os.path.normpath("/foo/bar baz/quux"),
+                          schema_salad.ref_resolver.uri_file_path("file:///foo/bar%20baz/quux"))
+        self.assertEquals("file:///foo/bar%20baz/quux%23zing%20zong", schema_salad.ref_resolver.file_uri("/foo/bar baz/quux#zing zong"))
+        self.assertEquals("file:///foo/bar%20baz/quux#zing%20zong", schema_salad.ref_resolver.file_uri("/foo/bar baz/quux#zing zong", split_frag=True))
+        self.assertEquals(os.path.normpath("/foo/bar baz/quux#zing zong"),
+                          schema_salad.ref_resolver.uri_file_path("file:///foo/bar%20baz/quux#zing%20zong"))
+
+
+class SourceLineTest(unittest.TestCase):
+    def test_sourceline(self):
+        ldr = schema_salad.ref_resolver.Loader({"id": "@id"})
+        b, _ = ldr.resolve_ref(get_data("tests/frag.yml"))
+
+        class TestExp(Exception):
+            pass
+
+        try:
+            with SourceLine(b, 1, TestExp, False):
+                raise Exception("Whoops")
+        except TestExp as e:
+            self.assertTrue(str(e).endswith("frag.yml:3:3: Whoops"))
+        except Exception:
+            self.assertFail()
+
+        try:
+            with SourceLine(b, 1, TestExp, True):
+                raise Exception("Whoops")
+        except TestExp as e:
+            self.assertTrue(str(e).splitlines()[0].endswith("frag.yml:3:3: Traceback (most recent call last):"))
+        except Exception:
+            self.assertFail()
 
 
 if __name__ == '__main__':
