@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-
+import os
 from typing import Any, Dict, List
 
 
@@ -27,15 +27,20 @@ def flatten(l, ltypes=(list, tuple)):
         return [l]
 
     ltype = type(l)
-    l = list(l)
+    lst = list(l)
     i = 0
-    while i < len(l):
-        while isinstance(l[i], ltypes):
-            if not l[i]:
-                l.pop(i)
+    while i < len(lst):
+        while isinstance(lst[i], ltypes):
+            if not lst[i]:
+                lst.pop(i)
                 i -= 1
                 break
             else:
-                l[i:i + 1] = l[i]
+                lst[i:i + 1] = lst[i]
         i += 1
-    return ltype(l)
+    return ltype(lst)
+
+# Check if we are on windows OS
+def onWindows():
+    # type: () -> (bool)
+    return os.name == 'nt'
