@@ -213,7 +213,8 @@ def validate_ex(expected_schema,                  # type: Schema
                 continue
             elif isinstance(datum, dict) and not isinstance(s, avro.schema.RecordSchema):
                 continue
-            elif isinstance(datum, (bool, six.integer_types, float, six.string_types)) and isinstance(s, (avro.schema.ArraySchema, avro.schema.RecordSchema)):
+            elif (isinstance(datum, (bool, six.integer_types, float, six.string_types)) and  # type: ignore
+                  isinstance(s, (avro.schema.ArraySchema, avro.schema.RecordSchema))):
                 continue
             elif datum is not None and s.type == "null":
                 continue
